from bolt import Runtime
from beet import Context
from beet.core.utils import required_field
from mecha import Mecha, rule, AstCommandSentinel
from dataclasses import dataclass
from textwrap import TextWrapper, dedent
import re
from itertools import chain

__all__ = ["Comment", "AstExplicitComment", "serialise_comment", "comment"]

comment_prefix = "# "
wrapper = TextWrapper(initial_indent=comment_prefix, subsequent_indent=comment_prefix)

paragraph_pattern = re.compile(
    r"""
    \r?\n
    [^\S\r\n]*  # ignore whitespaces other than newlines, not(not \s or \r or \n) == \s and not \r and not \n
    \r?\n
    """,
    flags=re.VERBOSE,
)


@dataclass(frozen=True)
class AstExplicitComment(AstCommandSentinel):
    comment: str = required_field()


@rule(AstExplicitComment)
def serialise_comment(node: AstExplicitComment, result: list[str]):
    paragraphs = (
        wrapper.wrap(dedent(paragraph))
        for paragraph in re.split(paragraph_pattern, node.comment)
    )
    lines = chain.from_iterable(paragraphs)
    result.append("\n".join(lines))


def beet_default(ctx: Context):
    ctx.require(comment)


def comment(ctx: Context):
    runtime = ctx.inject(Runtime)
    comment = ctx.inject(Comment)
    runtime.expose("comment", comment)


class Comment:
    _runtime: Runtime

    def __init__(self, ctx: Context):
        self._runtime = ctx.inject(Runtime)
        ctx.inject(Mecha).serialize.extend(serialise_comment)

    def __call__(self, comment: str):
        self._runtime.commands.append(AstExplicitComment(comment=comment))



from enum import Enum, auto


class BranchStatus(Enum):
    Constructed = auto()
    LoneBranch = auto()
    DoubleBranch = auto()


class EntityLoopStatus(Enum):
    NoLoop = auto()
    CollectingBody = auto()
    Deferred = auto()


class EntityLoopBreakStatus(Enum):
    All = auto()
    Some = auto()
    Single = auto()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bolt_raycast']

package_data = \
{'': ['*'], 'bolt_raycast': ['modules/*']}

install_requires = \
['beet>=0.80.1,<0.81.0', 'bolt>=0.21.0,<0.22.0', 'mecha>=0.61.0,<0.62.0']

setup_kwargs = {
    'name': 'bolt-raycast',
    'version': '0.1.0',
    'description': 'Easily generate raycasting functions for your datapack using bolt',
    'long_description': '# bolt-raycast\n\n[![GitHub Actions](https://github.com/vdvman1/bolt-raycast/workflows/CI/badge.svg)](https://github.com/vdvman1/bolt-raycast/actions)\n\n> Easily generate raycasting functions for your datapack using bolt\n\n## Installation\n\nThe package can be installed with `pip`. Note, you must have `beet`, `mecha` and `bolt` installed to use this package.\n\n```bash\npip install bolt_raycast\n```\n\n---\n\nLicense - [MIT](https://github.com/vdvman1/bolt-raycast/blob/main/LICENSE)\n\n',
    'author': 'Stefan van der Velden',
    'author_email': 'vdvman1@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/vdvman1/bolt-raycast',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



__version__ = "0.1.0"


from beet import Context
from beet.contrib.load import load


def beet_default(ctx: Context):
    ctx.require(
        load(
            data_pack={
                "data/bolt_raycast/modules": "@bolt_raycast/modules",
            },
        ),
    )
